#include <epplet.h>
#include <cdaudio.h>

#define PACKAGE "E-PlayCD"
#define VERSION "0.3"
#define NUM_BUTTONS 10
#define MAX_LBL_LEN 12
#define SEPARATOR "-"


Epplet_gadget cddb_label = NULL;
Epplet_gadget config_popup = NULL;
Epplet_gadget tracks_popup = NULL;
Epplet_gadget volume_slider = NULL;
char cddb_string[255];
struct disc_data DiscData;
struct disc_info DiscInfo;
int cd_desc;
int current_track;
struct disc_volume cd_volume;
int verbosity=1;

static void cb_close (void *data);
static void scroll_cddb_label (void *data);
int cddb_update (void *data);
void update_cddb_string (void);
void update_cddb_tracks (void);

/* Bounce Label left to right */
static void
bounce_cddb_label (void *data)
{
  static int start = 0;
  static int direction = 1;
  char saved_char = cddb_string[start + MAX_LBL_LEN];

  if (((start + MAX_LBL_LEN) > strlen (cddb_string)) || start < 0)
    direction = -direction;

  cddb_string[start + MAX_LBL_LEN] = '\0';
  Epplet_change_label (cddb_label, &cddb_string[start]);

  cddb_string[start + MAX_LBL_LEN] = saved_char;
  start += direction;

  Epplet_timer (bounce_cddb_label, NULL, 0.3, "TIMER");
}

static void
scroll_cddb_label (void *data)
{
  char *temp = (char *) data;
  int length = strlen (temp);
  char saved_char = temp[MAX_LBL_LEN];

  if (strlen (temp) < MAX_LBL_LEN)
    {
      Epplet_timer (scroll_cddb_label, &cddb_string, 0.3, "Scroller");
      return;
    }
  else
    {
      temp[MAX_LBL_LEN] = '\0';
      Epplet_change_label (cddb_label, temp);
      temp[MAX_LBL_LEN] = saved_char;
      saved_char = temp[0];
      memcpy (data, temp + 1, (size_t) length);
      temp[length - 1] = saved_char;
    }

  Epplet_timer (scroll_cddb_label, &cddb_string, 0.3, "Scroller");
}

/* Close */
static void
cb_close (void *data)
{
  Esync ();
  exit (0);
}

/* Rewind */
void
rew_cd (void *data)
{
	struct disc_timeval time;
	
	eplaycd_cd_stat(cd_desc, &DiscInfo);

	time.minutes = 0;
	time.seconds = -2;
	time.frames  = 0;
	cd_advance(cd_desc, time);
}

/* Fast Forward */
void
ff_cd (void *data)
{
	struct disc_timeval time;
	
	eplaycd_cd_stat(cd_desc, &DiscInfo);

	time.minutes = 0;
	time.seconds = 2;
	time.frames  = 0;

	cd_advance(cd_desc, time); 
}

/* Play */
void
play_cd (void *data)
{
	int* the_track = (int *)data;
	
  eplaycd_cd_stat (cd_desc, &DiscInfo);

  if(the_track != NULL) 
	  cd_play (cd_desc, *the_track);
  else if (DiscInfo.disc_present)
    {

      switch (DiscInfo.disc_mode)
	{
	case CDAUDIO_PLAYING:
	  cd_pause (cd_desc);
	  break;
	case CDAUDIO_PAUSED:
	  cd_resume (cd_desc);
	  break;
	default:
	  cd_play (cd_desc, DiscInfo.disc_first_track);
	  break;
	}
    }

  cddb_update (NULL);
}

/* Stop */
void
stop_cd (void *data)
{
  if (eplaycd_cd_stat (cd_desc, &DiscInfo) < 0)
    printf ("WHAT THE FUCK!\n");

  switch (DiscInfo.disc_mode)
    {
    case CDAUDIO_PLAYING:
    case CDAUDIO_PAUSED:
      cd_stop (cd_desc);
      break;
    case CDAUDIO_NOSTATUS:
    case CDAUDIO_COMPLETED:
      if (DiscInfo.disc_present)
	{
	  cd_eject (cd_desc);
	}
      else
	{
	  cd_close (cd_desc);
	  cddb_update (NULL);
	}
      break;
    default:
      cd_close (cd_desc);
      cddb_update (NULL);
      break;
    }
}

/* Previous Track */
void
prev_track (void *data)
{
  eplaycd_cd_stat (cd_desc, &DiscInfo);

  if (DiscInfo.disc_current_track <= DiscInfo.disc_first_track)
    cd_play (cd_desc, DiscInfo.disc_total_tracks);
  else
    cd_play (cd_desc, DiscInfo.disc_current_track - 1);

  update_cddb_string ();
}

/* Next Track */
void
next_track (void *data)
{
  eplaycd_cd_stat (cd_desc, &DiscInfo);

  if (DiscInfo.disc_current_track >= DiscInfo.disc_total_tracks)
    cd_play (cd_desc, DiscInfo.disc_first_track);
  else
    cd_play (cd_desc, DiscInfo.disc_current_track + 1);

  update_cddb_string ();
}

/* Stat CD */
int
eplaycd_cd_stat (int cd_desc, struct disc_info *disc)
{
  cd_stat (cd_desc, disc);
  if (!disc->disc_present)
    {
      cd_close (cd_desc);
      cd_stat (cd_desc, disc);
      if (!disc->disc_present)
	{
	  puts ("No disc in drive");
	  return -1;
	}
    }

  return 0;
}

/* Update CDDB Info */
int
cddb_update (void *data)
{
	if (cddb_read_data( cd_desc, &DiscData ) < 0)
		return -1;

	update_cddb_string();
	update_cddb_tracks();

	return 0;
}

/* Update String for CDDB label */
void
update_cddb_string (void)
{
  eplaycd_cd_stat (cd_desc, &DiscInfo);

  strcpy (cddb_string, DiscData.data_artist);
  strcat (cddb_string, SEPARATOR);
  strcat (cddb_string, DiscData.data_title);
  strcat (cddb_string, SEPARATOR);
  strcat (cddb_string,
	  DiscData.data_track[DiscInfo.disc_current_track].track_name);
  strcat (cddb_string, "...");
}

void
update_cddb_tracks (void)
{
	int i;
	static int* tracks = NULL;

	free(tracks);

	tracks = (int *)calloc((size_t)DiscInfo.disc_total_tracks, sizeof(int));

	//Epplet_gadget_destroy(tracks_popup);
	tracks_popup = Epplet_create_popup();
	
	Epplet_add_popup_entry(tracks_popup, 
			"-NeverMind-", 
			NULL, NULL, NULL);

	for(i=0; i < DiscInfo.disc_total_tracks ; i++)
	{
		tracks[i] = i+1;

		Epplet_add_popup_entry(tracks_popup,
				DiscData.data_track[i].track_name, 
				NULL, play_cd, &tracks[i]);
	}
}

/* Pop-Up About Box */
void
do_help (void *data)
{
  Epplet_show_about ("E-PlayCD");
}

void
popup_config(void *data)
{
	Epplet_pop_popup(config_popup, 0);
}

void
popup_tracks(void *data)
{
	Epplet_pop_popup(tracks_popup, 0);
}

/* Set the Volume */
void
set_volume(void *data)
{
	struct disc_volume *the_volume = (struct disc_volume *)data;

	the_volume->vol_front.right = the_volume->vol_front.left;
	cd_set_volume(cd_desc, *the_volume);
}

/* Get the Volume */
void
get_volume(void)
{
	cd_get_volume(cd_desc, &cd_volume);
}

/* MAIN */
int
main (int argc, char **argv)
{
  int i;
  int start_vol = 0;

  char *button_stds[] = { 
	  "CLOSE", "PLAY", "STOP", "PREVIOUS", "NEXT", "ARROW_UP", 
	  "REPEAT", "REWIND", "FAST_FORWARD", "SKIP", "CONFIGURE", "HELP"
  };

  void *button_funcs[] = { 
	  cb_close, play_cd, stop_cd, prev_track, next_track, popup_tracks, 
	  NULL, rew_cd, ff_cd, NULL, cddb_update, do_help 
  };

  atexit(Epplet_cleanup);
  Epplet_Init ("E-CDPlay", "0.1", "Enlightenment CD Player Epplet",
	       5, 3, argc, argv, 0);

  printf ("Initializing /dev/cdrom...");
  if ((cd_desc = cd_init_device ("/dev/cdrom")) < 0)
    {
      printf ("error\n");
      exit (0);
    }
  printf ("done\n");

  Epplet_timer (scroll_cddb_label, &cddb_string, 0.5, "Scroller");

  for (i = 0; i < 6; i++)
  {
       	  Epplet_gadget_show (Epplet_create_button (NULL, NULL,
				  2 + (i * 13), 2, 0, 0,
				  button_stds[i], 0, NULL,
				  button_funcs[i], NULL));
  }

  for (i = 0; i < 6; i++)
  {
	  Epplet_gadget_show (Epplet_create_button (NULL, NULL,
				  2 + (i * 13), 15, 0, 0,
				  button_stds[6+i], 0, NULL,
				  button_funcs[6+i], NULL));
  }

  tracks_popup = Epplet_create_popup();
  Epplet_add_popup_entry(tracks_popup, "-NeverMind-", NULL, NULL, NULL);

  
  get_volume();
  Epplet_gadget_show(
		     volume_slider = 
		     Epplet_create_hslider(2, 27, 76, 0, 255, 4, 8,
					   &cd_volume.vol_front.left,
					   set_volume,
					   &cd_volume));

  Epplet_gadget_show (cddb_label =
                      Epplet_create_label (2, 35, "No CDDB Data", 2));
				   

  Epplet_show ();
  Epplet_Loop ();
  return 0;
}
